@@ -1,6 +1,6 @@
 """This module contains the InstaClient class"""
 from logging import log
-from os import waitpid
+from os import error, waitpid
 from selenium import webdriver
 from selenium.webdriver.remote.webelement import WebElement
 from selenium.webdriver.support.expected_conditions import presence_of_element_located
@@ -20,7 +20,7 @@ class InstaClient:
     CHROMEDRIVER=1
     LOCAHOST=1
     WEB_SERVER=2
-    def __init__(self, driver_type: int=CHROMEDRIVER, host_type:int=LOCAHOST, driver_path=None):
+    def __init__(self, driver_type: int=CHROMEDRIVER, host_type:int=LOCAHOST, driver_path=None, error_callback=None):
         """
         Create an `InstaClient` object to access the instagram website.
 
@@ -29,11 +29,13 @@ class InstaClient:
             driver_type (int, optional): The type of browser driver to run instagram on. Defaults to CHROMEDRIVER.
             host_type (int, optional): Whether the code is run locally or on a server. Defaults to LOCAHOST.
             driver_path (str): The path where you saved the c`hromedriver.exe` file. This is required if you are running the client locally. Defaults to None
+            error_callback (callback): A callback method to be called when an error occures within the InstaClient. Your custom error_callbak must require only one argument named `driver`: a driver like object (The InstaClient will pass itself to the method as `driver` argument)
 
         Raises:
             InvaildHostError: Raised if host int does not correspond to any host type
             InvaildDriverError: Raised if driver int does not correspond to any driver type.
             error: Normal Exception, raised if anything fails when creating the client.
+            InvalidErrorCallbackError: Raised if the `error_callback` is not callable
         """
         
         self.driver_type = driver_type
@@ -41,6 +43,10 @@ class InstaClient:
         if host_type == self.LOCAHOST and driver_path is None:
             raise InvalidDriverPathError(driver_path)
         self.driver_path = driver_path
+        if error_callback:
+            if not callable(error_callback):
+                raise InvalidErrorCallbackError()
+        self.error_callback = error_callback
         self.logged_in = False
         self.driver = None
         self.username = None
@@ -57,12 +63,19 @@ class InstaClient:
         Returns:
             bool: True if client is logged in, False if client is not connected or webdriver is not open.
         """
+        print('INSTACLIENT: Check Status')
         if not self.driver:
             return False
-        self.driver.get(ClientUrls.HOME_URL)
+        print(self.driver.current_url)
+        if ClientUrls.HOME_URL not in self.driver.current_url:
+            self.driver.get(ClientUrls.HOME_URL)
         if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.COOKIES_LINK))):
             self.__dismiss_cookies()
-            
+        if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.NOT_NOW_BTN))):
+            btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.NOT_NOW_BTN)))
+            btn.click()
+            print('INSTACLIENT: Dismissed dialogue')
+
         icon = self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.NAV_BAR)), wait_time=4)
         if icon:
             self.logged_in = True
@@ -187,12 +200,8 @@ class InstaClient:
             self.driver.get(ClientUrls.HOME_URL)
 
             # Detect 'Turn On Notifications' Box
-            try:
-                no_notifications_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.NO_NOTIFICATIONS_BTN)), wait_time=3, url=ClientUrls.HOME_URL)
-                no_notifications_btn.click()
-            except:
-                pass
-            self.__dismiss_dialogue()
+            if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.NO_NOTIFICATIONS_BTN))):
+                self.__dismiss_dialogue()
         return self.logged_in
 
 
@@ -569,18 +578,22 @@ class InstaClient:
         Returns:
             bool: True if the 
         """
+        print('INSTACLIENT: LOGOUT')
         result = self.check_status()
         if result:
             if discard_driver:
                 self.__discard_driver()
+                print('INSTACLIENT: Logged Out')
             else:
                 self.driver.get(ClientUrls.NAV_USER.format(self.username))
+                time.sleep(1)
                 settings_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.SETTINGS_BTN)), wait_time=4)
                 settings_btn.click()
                 logout_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.LOG_OUT_BTN)), wait_time=4)
                 logout_btn.click()
                 confirm_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.CONFIRM_LOGOUT_BTN)), wait_time=4)
                 confirm_btn.click()
+                print('INSTACLIENT: Logged Out')
             return True
         else:
             return True
@@ -723,10 +736,15 @@ class InstaClient:
             return widgets
         except TimeoutException:
             # Element was not found in time
+            print('INSTACLIENT: Element Not Found...')
             if attempt < 1:
                 if self.__check_existence(EC.presence_of_element_located((By.XPATH, Paths.COOKIES_LINK))):
                     self.__dismiss_cookies()
                 if not self.check_status():
+                    if url in self.driver.current_url:
+                        self.driver.get(url)
+                        self.__find_element(expectation, url, wait_time, attempt+1)
+                        time.sleep(1)
                     # Not Logged In!
                     raise NotLoggedInError()
                 else:
@@ -734,6 +752,8 @@ class InstaClient:
                         self.driver.get(url)
                         self.__find_element(expectation, url, wait_time, attempt+1)
                     else:
+                        if self.error_callback:
+                            self.error_callback(self.driver)
                         raise NoSuchElementException()
             else:
                 raise NoSuchElementException()
@@ -801,6 +821,8 @@ class InstaClient:
     def __dismiss_cookies(self):
         accept_btn = self.__find_element(EC.presence_of_element_located((By.XPATH, Paths.ACCEPT_COOKIES)))
         accept_btn.click()
+        print('INSTACLIENT: Dismissed Cookies')
+
 
     def __dismiss_dialogue(self):
         """
