@@ -3,5 +3,5 @@ class ClientUrls:
     NAV_USER='https://www.instagram.com/{}/'
     SEARCH_TAGS='https://www.instagram.com/explore/tags/{}/'
     FOLLOWERS_URL = 'https://www.instagram.com/{}/followers/'
-    HOME_URL =  'https://instagram.com/'
+    HOME_URL =  'https://www.instagram.com/'
     SECURITY_CODE_URL = 'https://www.instagram.com/challenge/'
\ No newline at end of file
